@@ -1,7 +1,11 @@
 #include "vtkF3DPointSplatMapper.h"
 
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
 #include "vtkF3DBitonicSort.h"
 #include "vtkF3DComputeDepthCS.h"
+#endif
+#include "vtkF3DPointSplatVS.h"
+#include "vtkF3DRenderer.h"
 
 #include <vtkCamera.h>
 #include <vtkObjectFactory.h>
@@ -12,13 +16,16 @@
 #include <vtkOpenGLRenderer.h>
 #include <vtkOpenGLShaderCache.h>
 #include <vtkOpenGLState.h>
+#include <vtkOpenGLVertexArrayObject.h>
 #include <vtkOpenGLVertexBufferObject.h>
 #include <vtkOpenGLVertexBufferObjectGroup.h>
 #include <vtkPointData.h>
 #include <vtkPolyData.h>
 #include <vtkShader.h>
 #include <vtkShaderProgram.h>
+#include <vtkShaderProperty.h>
 #include <vtkTextureObject.h>
+#include <vtkUniforms.h>
 #include <vtkVersion.h>
 
 #if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 3, 20240914)
@@ -42,12 +49,19 @@ public:
 protected:
   vtkF3DSplatMapperHelper();
 
+  void GetShaderTemplate(
+    std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor) override;
+
   // overridden to create the OpenGL depth buffer
   void BuildBufferObjects(vtkRenderer* ren, vtkActor* act) override;
 
   // overridden to sort splats
   void RenderPieceDraw(vtkRenderer* ren, vtkActor* act) override;
 
+  // add instance id and TAA to fragment shader
+  void ReplaceShaderPositionVC(
+    std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor) override;
+
   // add spherical harmonics
   void ReplaceShaderColor(
     std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor) override;
@@ -61,6 +75,7 @@ protected:
     vtkOpenGLHelper& cellBO, vtkRenderer* ren, vtkActor* actor) override;
 
 private:
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
   void SortSplats(vtkRenderer* ren);
 
   vtkNew<vtkShader> DepthComputeShader;
@@ -71,6 +86,9 @@ private:
 
   double DirectionThreshold = 0.999;
   double LastDirection[3] = { 0.0, 0.0, 0.0 };
+#endif
+
+  bool OwnerUseInstancing();
 
   int MaxTextureSize = 0;
   vtkNew<vtkTextureObject> SphericalHarmonicsTexture;
@@ -83,11 +101,30 @@ vtkStandardNewMacro(vtkF3DSplatMapperHelper);
 //----------------------------------------------------------------------------
 vtkF3DSplatMapperHelper::vtkF3DSplatMapperHelper()
 {
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
   this->DepthComputeShader->SetType(vtkShader::Compute);
   this->DepthComputeShader->SetSource(vtkF3DComputeDepthCS);
   this->DepthProgram->SetComputeShader(this->DepthComputeShader);
 
   this->Sorter->Initialize(512, VTK_FLOAT, VTK_UNSIGNED_INT);
+#endif
+}
+
+//----------------------------------------------------------------------------
+void vtkF3DSplatMapperHelper::GetShaderTemplate(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
+{
+  if (this->OwnerUseInstancing())
+  {
+    this->vtkOpenGLPolyDataMapper::GetShaderTemplate(shaders, ren, actor);
+
+    this->UsingPoints = false;
+    shaders[vtkShader::Vertex]->SetSource(vtkF3DPointSplatVS);
+  }
+  else
+  {
+    this->Superclass::GetShaderTemplate(shaders, ren, actor);
+  }
 }
 
 //----------------------------------------------------------------------------
@@ -104,9 +141,11 @@ void vtkF3DSplatMapperHelper::BuildBufferObjects(vtkRenderer* ren, vtkActor* act
 
   vtkOpenGLPointGaussianMapperHelper::BuildBufferObjects(ren, act);
 
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
   // allocate a buffer of depths used for sorting splats
   this->DepthBuffer->Allocate(splatCount * sizeof(float), vtkOpenGLBufferObject::ArrayBuffer,
     vtkOpenGLBufferObject::DynamicCopy);
+#endif
 
   this->SphericalHarmonicsDegree = 0;
 
@@ -252,6 +291,10 @@ void vtkF3DSplatMapperHelper::SetMapperShaderParameters(
       "sphericalHarmonics", this->SphericalHarmonicsTexture->GetTextureUnit());
   }
 
+#if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 5, 20251120)
+  this->VBOs->SetInstancing(this->OwnerUseInstancing());
+#endif
+
   this->Superclass::SetMapperShaderParameters(cellBO, ren, actor);
 }
 
@@ -268,6 +311,7 @@ void vtkF3DSplatMapperHelper::SetCameraShaderParameters(
   this->Superclass::SetCameraShaderParameters(cellBO, ren, actor);
 }
 
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
 //----------------------------------------------------------------------------
 void vtkF3DSplatMapperHelper::SortSplats(vtkRenderer* ren)
 {
@@ -318,16 +362,118 @@ void vtkF3DSplatMapperHelper::SortSplats(vtkRenderer* ren)
     }
   }
 }
+#endif
 
 //----------------------------------------------------------------------------
 void vtkF3DSplatMapperHelper::RenderPieceDraw(vtkRenderer* ren, vtkActor* actor)
 {
-  if (vtkShader::IsComputeShaderSupported() && actor->GetForceTranslucent())
+#if !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
+  const vtkF3DRenderer* renderer = vtkF3DRenderer::SafeDownCast(ren);
+
+  if (renderer->GetBlendingMode() == vtkF3DRenderer::BlendingMode::SORT &&
+    vtkShader::IsComputeShaderSupported() && actor->HasTranslucentPolygonalGeometry())
   {
     this->SortSplats(ren);
   }
+#endif
 
-  vtkOpenGLPointGaussianMapperHelper::RenderPieceDraw(ren, actor);
+  if (this->OwnerUseInstancing())
+  {
+    int numVerts = this->VBOs->GetNumberOfTuples("vertexMC");
+    if (numVerts)
+    {
+      this->UpdateShaders(this->Primitives[PrimitivePoints], ren, actor);
+
+      this->Primitives[PrimitivePoints].VAO->Bind();
+      glDrawArraysInstanced(GL_TRIANGLE_STRIP, 0, 4, numVerts);
+      this->Primitives[PrimitivePoints].VAO->Release();
+    }
+  }
+  else
+  {
+    // Use VTK geometry shader based rendering
+    vtkOpenGLPointGaussianMapperHelper::RenderPieceDraw(ren, actor);
+  }
+}
+
+//----------------------------------------------------------------------------
+void vtkF3DSplatMapperHelper::ReplaceShaderPositionVC(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
+{
+  vtkUniforms* uniforms = actor->GetShaderProperty()->GetVertexCustomUniforms();
+
+  // TAA
+  vtkUniforms::TupleType type = uniforms->GetUniformTupleType("jitter");
+  if (type != vtkUniforms::TupleTypeInvalid)
+  {
+    std::string VSSource = shaders[vtkShader::Vertex]->GetSource();
+
+    vtkShaderProgram::Substitute(VSSource, "//VTK::PositionVC::Dec",
+      "//VTK::PositionVC::Dec\n"
+      "  uniform vec2 jitter;\n",
+      false);
+
+    vtkShaderProgram::Substitute(VSSource, "//VTK::PositionVC::Impl",
+      "//VTK::PositionVC::Impl\n"
+      "  // apply temporal jittering for TAA\n"
+      "  gl_Position.xy += jitter * gl_Position.w;\n",
+      false);
+
+    shaders[vtkShader::Vertex]->SetSource(VSSource);
+  }
+
+  if (this->OwnerUseInstancing())
+  {
+    std::string FSSource = shaders[vtkShader::Fragment]->GetSource();
+
+    vtkShaderProgram::Substitute(FSSource, "//VTK::PositionVC::Dec\n",
+      "//VTK::PositionVC::Dec\n"
+      "flat in int instanceId;\n");
+
+    shaders[vtkShader::Fragment]->SetSource(FSSource);
+  }
+
+  std::string VSSource = shaders[vtkShader::Vertex]->GetSource();
+
+  if (this->OwnerUseInstancing())
+  {
+    vtkShaderProgram::Substitute(VSSource, "//VTK::PositionVC::Dec\n",
+      "uniform float boundScale;\n"
+      "out vec2 offsetVCVSOutput;\n"
+      "flat out int instanceId;\n");
+
+    vtkShaderProgram::Substitute(VSSource, "//VTK::PositionVC::Impl\n",
+      R"(
+  vec2 offsets[4] = vec2[](vec2(-boundScale, -boundScale),
+                           vec2(boundScale, -boundScale),
+                           vec2(-boundScale, boundScale),
+                           vec2(boundScale, boundScale));
+
+  offsetVCVSOutput = offsets[gl_VertexID % 4];
+
+  vec4 posNDC = posDC;
+  posNDC = posNDC / posNDC.w;
+
+  gl_Position = vec4(posNDC.xy + transform * offsetVCVSOutput, posNDC.zw);
+
+  instanceId = gl_InstanceID;
+)");
+  }
+  else
+  {
+    vtkShaderProgram::Substitute(
+      VSSource, "//VTK::PositionVC::Dec\n", "out mat2 transformVCVSOutput;\n");
+
+    vtkShaderProgram::Substitute(VSSource, "//VTK::PositionVC::Impl\n",
+      R"(
+  transformVCVSOutput = transform;
+  gl_Position = posDC;
+)");
+  }
+
+  shaders[vtkShader::Vertex]->SetSource(VSSource);
+
+  this->Superclass::ReplaceShaderPositionVC(shaders, ren, actor);
 }
 
 //----------------------------------------------------------------------------
@@ -338,16 +484,19 @@ void vtkF3DSplatMapperHelper::ReplaceShaderColor(
   {
     std::string VSSource = shaders[vtkShader::Vertex]->GetSource();
 
-    vtkShaderProgram::Substitute(VSSource, "//VTK::Color::Dec",
-      "//VTK::Color::Dec\n\n"
-      "uniform sampler2DArray sphericalHarmonics;\n"
-      "uniform vec3 cameraDirection;\n"
-      "vec3 decode(ivec3 texelIndex)\n"
-      "{\n"
-      "  vec3 texel = texelFetch(sphericalHarmonics, texelIndex, 0).rgb;\n"
-      "  return texel * 2.0 - 1.0;\n"
-      "}\n\n",
-      false);
+    if (this->SphericalHarmonicsDegree > 0)
+    {
+      vtkShaderProgram::Substitute(VSSource, "//VTK::Color::Dec",
+        "//VTK::Color::Dec\n\n"
+        "uniform sampler2DArray sphericalHarmonics;\n"
+        "uniform vec3 cameraDirection;\n"
+        "vec3 decode(ivec3 texelIndex)\n"
+        "{\n"
+        "  vec3 texel = texelFetch(sphericalHarmonics, texelIndex, 0).rgb;\n"
+        "  return texel * 2.0 - 1.0;\n"
+        "}\n\n",
+        false);
+    }
 
     std::stringstream shStr;
     shStr << "//VTK::Color::Impl\n";
@@ -355,8 +504,18 @@ void vtkF3DSplatMapperHelper::ReplaceShaderColor(
     if (this->SphericalHarmonicsDegree >= 1)
     {
       shStr << "  vec3 sh1 = vec3(0);\n";
-      shStr << "  ivec2 texelIndex = ivec2(gl_VertexID % " << this->MaxTextureSize
-            << ", gl_VertexID / " << this->MaxTextureSize << ");\n";
+
+      if (this->OwnerUseInstancing())
+      {
+        shStr << "  ivec2 texelIndex = ivec2(gl_InstanceID % " << this->MaxTextureSize
+              << ", gl_InstanceID / " << this->MaxTextureSize << ");\n";
+      }
+      else
+      {
+        shStr << "  ivec2 texelIndex = ivec2(gl_VertexID % " << this->MaxTextureSize
+              << ", gl_VertexID / " << this->MaxTextureSize << ");\n";
+      }
+
       shStr << "  sh1 -= 0.48860251 * decode(ivec3(texelIndex, 0)) * cameraDirection.y;\n";
       shStr << "  sh1 += 0.48860251 * decode(ivec3(texelIndex, 1)) * cameraDirection.z;\n";
       shStr << "  sh1 -= 0.48860251 * decode(ivec3(texelIndex, 2)) * cameraDirection.x;\n";
@@ -416,6 +575,18 @@ void vtkF3DSplatMapperHelper::ReplaceShaderColor(
   this->Superclass::ReplaceShaderColor(shaders, ren, actor);
 }
 
+//----------------------------------------------------------------------------
+bool vtkF3DSplatMapperHelper::OwnerUseInstancing()
+{
+  bool useInstancing = false;
+
+#if VTK_VERSION_NUMBER >= VTK_VERSION_CHECK(9, 5, 20251120)
+  useInstancing = vtkF3DPointSplatMapper::SafeDownCast(this->Owner)->GetUseInstancing();
+#endif
+
+  return useInstancing;
+}
+
 //----------------------------------------------------------------------------
 vtkOpenGLPointGaussianMapperHelper* vtkF3DPointSplatMapper::CreateHelper()
 {
